@@ -12,20 +12,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import testscenarios
 import time
 
-from oslo_log import log as logging
-
 from tempest.common import waiters
 from tempest import config
 from tempest.lib.common.utils import test_utils
+from tempest.lib import decorators
 from tempest import test
+import testscenarios
 
 from networking_sfc.tests.tempest_plugin.tests.scenario import base
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
@@ -38,7 +36,7 @@ class TestSfc(base.SfcScenarioTest):
             msg = ('Either project_networks_reachable must be "true", or '
                    'public_network_id must be defined.')
             raise cls.skipException(msg)
-        required_exts = ['sfc', 'flowclassifier']
+        required_exts = ['sfc', 'flow_classifier']
         for ext in required_exts:
             if not test.is_extension_enabled(ext, 'network'):
                 msg = "%s Extension not enabled." % ext
@@ -50,14 +48,18 @@ class TestSfc(base.SfcScenarioTest):
         cls.set_network_resources()
         super(TestSfc, cls).setup_credentials()
 
+    def _setup_security_group(self):
+        self.security_group = self._create_security_group()
+        self._create_security_group_rule(
+            self.security_group, protocol=None,
+            direction='ingress'
+        )
+
     def setUp(self):
         super(TestSfc, self).setUp()
         self.ssh_user = CONF.validation.image_ssh_user
         self.keypair = self.create_keypair()
-        self.security_group = self._create_security_group()
-        self._create_security_group_rule(
-            self.security_group, protocol=None,
-            direction='ingress')
+        self._setup_security_group()
         self.net1, self.subnet1, self.router1 = self.create_networks()
         self.router2 = self._create_router()
         self.router3 = self._create_router()
@@ -94,9 +96,67 @@ class TestSfc(base.SfcScenarioTest):
     def _wait_for_port_chain_status(self, port_chain, status):
         time.sleep(10)
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a8')
+    def _create_port_chain_helper(self, symmetric):
+        (
+            server1_floating_ip, server1_port_id, server1_fixed_ip
+        ) = self._setup_server(self.net1)
+        (
+            server2_floating_ip, server2_port_id, server2_fixed_ip
+        ) = self._setup_server(self.net1)
+        self._check_connectivity(
+            server1_floating_ip, server2_fixed_ip,
+            [],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
+        if symmetric:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                logical_destination_port=server2_port_id,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
+        else:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
+        port_pair = self._create_port_pair(
+            ingress=self.router2_net1['id'],
+            egress=self.router2_net1['id']
+        )
+        port_pair_group = self._create_port_pair_group(
+            port_pairs=[port_pair['id']]
+        )
+        port_chain = self._create_port_chain(
+            port_pair_groups=[port_pair_group['id']],
+            flow_classifiers=[fc['id']],
+            chain_parameters={'symmetric': symmetric}
+        )
+        self._check_connectivity(
+            server1_floating_ip, server2_fixed_ip,
+            [[self.router2_net1_fixed_ip]],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
+        self.portchain_client.delete_port_chain(port_chain['id'])
+        self._wait_for_port_chain_status(port_chain, 'DELETED')
+        self._check_connectivity(
+            server1_floating_ip, server2_fixed_ip,
+            [],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
+
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a8')
     @test.services('compute', 'network')
     def test_create_port_chain(self):
+        self._create_port_chain_helper(False)
+
+    @decorators.idempotent_id('35927961-1904-4a6b-9d08-ad819f1cf812')
+    @test.services('compute', 'network')
+    def test_create_port_chain_symmetric(self):
+        self._create_port_chain_helper(True)
+
+    def _create_port_chain_multi_fc_helper(self, symmetric):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
         ) = self._setup_server(self.net1)
@@ -108,11 +168,35 @@ class TestSfc(base.SfcScenarioTest):
             [],
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
-        fc = self._create_flowclassifier(
-            logical_source_port=server1_port_id,
-            source_ip_prefix='%s/32' % server1_fixed_ip,
-            destination_ip_prefix='%s/32' % server2_fixed_ip
-        )
+        self._check_connectivity(
+            server2_floating_ip, server1_fixed_ip,
+            [],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
+        if symmetric:
+            fc1 = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                logical_destination_port=server2_port_id,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
+            fc2 = self._create_flowclassifier(
+                logical_source_port=server2_port_id,
+                source_ip_prefix='%s/32' % server2_fixed_ip,
+                logical_destination_port=server1_port_id,
+                destination_ip_prefix='%s/32' % server1_fixed_ip
+            )
+        else:
+            fc1 = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
+            fc2 = self._create_flowclassifier(
+                logical_source_port=server2_port_id,
+                source_ip_prefix='%s/32' % server2_fixed_ip,
+                destination_ip_prefix='%s/32' % server1_fixed_ip
+            )
         port_pair = self._create_port_pair(
             ingress=self.router2_net1['id'],
             egress=self.router2_net1['id']
@@ -122,13 +206,19 @@ class TestSfc(base.SfcScenarioTest):
         )
         port_chain = self._create_port_chain(
             port_pair_groups=[port_pair_group['id']],
-            flow_classifiers=[fc['id']]
+            flow_classifiers=[fc1['id'], fc2['id']],
+            chain_parameters={'symmetric': symmetric}
         )
         self._check_connectivity(
             server1_floating_ip, server2_fixed_ip,
             [[self.router2_net1_fixed_ip]],
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
+        self._check_connectivity(
+            server2_floating_ip, server1_fixed_ip,
+            [[self.router2_net1_fixed_ip]],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
         self.portchain_client.delete_port_chain(port_chain['id'])
         self._wait_for_port_chain_status(port_chain, 'DELETED')
         self._check_connectivity(
@@ -136,74 +226,23 @@ class TestSfc(base.SfcScenarioTest):
             [],
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
+        self._check_connectivity(
+            server2_floating_ip, server1_fixed_ip,
+            [],
+            username=self.ssh_user,
+            private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a9')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21a9')
     @test.services('compute', 'network')
     def test_create_port_chain_multi_flow_classifiers(self):
-        (
-            server1_floating_ip, server1_port_id, server1_fixed_ip
-        ) = self._setup_server(self.net1)
-        (
-            server2_floating_ip, server2_port_id, server2_fixed_ip
-        ) = self._setup_server(self.net1)
-        self._check_connectivity(
-            server1_floating_ip, server2_fixed_ip,
-            [],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-        self._check_connectivity(
-            server2_floating_ip, server1_fixed_ip,
-            [],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-        fc1 = self._create_flowclassifier(
-            logical_source_port=server1_port_id,
-            source_ip_prefix='%s/32' % server1_fixed_ip,
-            destination_ip_prefix='%s/32' % server2_fixed_ip
-        )
-        fc2 = self._create_flowclassifier(
-            logical_source_port=server2_port_id,
-            source_ip_prefix='%s/32' % server2_fixed_ip,
-            destination_ip_prefix='%s/32' % server1_fixed_ip
-        )
+        self._create_port_chain_multi_fc_helper(False)
 
-        port_pair = self._create_port_pair(
-            ingress=self.router2_net1['id'],
-            egress=self.router2_net1['id']
-        )
-        port_pair_group = self._create_port_pair_group(
-            port_pairs=[port_pair['id']]
-        )
-        port_chain = self._create_port_chain(
-            port_pair_groups=[port_pair_group['id']],
-            flow_classifiers=[fc1['id'], fc2['id']]
-        )
-        self._check_connectivity(
-            server1_floating_ip, server2_fixed_ip,
-            [[self.router2_net1_fixed_ip]],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-        self._check_connectivity(
-            server2_floating_ip, server1_fixed_ip,
-            [[self.router2_net1_fixed_ip]],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-        self.portchain_client.delete_port_chain(port_chain['id'])
-        self._wait_for_port_chain_status(port_chain, 'DELETED')
-        self._check_connectivity(
-            server1_floating_ip, server2_fixed_ip,
-            [],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-        self._check_connectivity(
-            server2_floating_ip, server1_fixed_ip,
-            [],
-            username=self.ssh_user,
-            private_key=self.keypair['private_key'])
-
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21aa')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b1')
     @test.services('compute', 'network')
-    def test_create_port_chain_multi_port_pairs(self):
+    def test_create_port_chain_multi_flow_classifiers_symmetric(self):
+        self._create_port_chain_multi_fc_helper(True)
+
+    def _create_port_chain_multi_port_pairs_helper(self, symmetric):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
         ) = self._setup_server(self.net1)
@@ -215,11 +254,19 @@ class TestSfc(base.SfcScenarioTest):
             [],
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
-        fc = self._create_flowclassifier(
-            logical_source_port=server1_port_id,
-            source_ip_prefix='%s/32' % server1_fixed_ip,
-            destination_ip_prefix='%s/32' % server2_fixed_ip
-        )
+        if symmetric:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                logical_destination_port=server2_port_id,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
+        else:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix='%s/32' % server1_fixed_ip,
+                destination_ip_prefix='%s/32' % server2_fixed_ip
+            )
         port_pair1 = self._create_port_pair(
             ingress=self.router2_net1['id'],
             egress=self.router2_net1['id']
@@ -233,7 +280,8 @@ class TestSfc(base.SfcScenarioTest):
         )
         port_chain = self._create_port_chain(
             port_pair_groups=[port_pair_group['id']],
-            flow_classifiers=[fc['id']]
+            flow_classifiers=[fc['id']],
+            chain_parameters={'symmetric': symmetric}
         )
         self._check_connectivity(
             server1_floating_ip, server2_fixed_ip,
@@ -248,9 +296,17 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ab')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21aa')
     @test.services('compute', 'network')
-    def test_create_port_chain_multi_port_pair_groups(self):
+    def test_create_port_chain_multi_port_pairs(self):
+        self._create_port_chain_multi_port_pairs_helper(False)
+
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f869be1e21ad')
+    @test.services('compute', 'network')
+    def test_create_port_chain_multi_port_pairs_symmetric(self):
+        self._create_port_chain_multi_port_pairs_helper(True)
+
+    def _create_port_chain_multi_ppg_helper(self, symmetric):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
         ) = self._setup_server(self.net1)
@@ -262,11 +318,19 @@ class TestSfc(base.SfcScenarioTest):
             [],
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
-        fc = self._create_flowclassifier(
-            logical_source_port=server1_port_id,
-            source_ip_prefix='%s/32' % server1_fixed_ip,
-            destination_ip_prefix='%s/32' % server2_fixed_ip
-        )
+        if symmetric:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix="%s/32" % server1_fixed_ip,
+                logical_destination_port=server2_port_id,
+                destination_ip_prefix="%s/32" % server2_fixed_ip
+            )
+        else:
+            fc = self._create_flowclassifier(
+                logical_source_port=server1_port_id,
+                source_ip_prefix="%s/32" % server1_fixed_ip,
+                destination_ip_prefix="%s/32" % server2_fixed_ip
+            )
         port_pair1 = self._create_port_pair(
             ingress=self.router2_net1['id'],
             egress=self.router2_net1['id']
@@ -283,7 +347,8 @@ class TestSfc(base.SfcScenarioTest):
         )
         port_chain = self._create_port_chain(
             port_pair_groups=[port_pair_group1['id'], port_pair_group2['id']],
-            flow_classifiers=[fc['id']]
+            flow_classifiers=[fc['id']],
+            chain_parameters={'symmetric': symmetric}
         )
         self._check_connectivity(
             server1_floating_ip, server2_fixed_ip,
@@ -298,7 +363,17 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e22ab')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ab')
+    @test.services('compute', 'network')
+    def test_create_port_chain_multi_port_pair_groups(self):
+        self._create_port_chain_multi_ppg_helper(False)
+
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b0')
+    @test.services('compute', 'network')
+    def test_create_port_chain_multi_port_pair_groups_symmetric(self):
+        self._create_port_chain_multi_ppg_helper(True)
+
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e22ab')
     @test.services('compute', 'network')
     def test_create_multi_port_chain(self):
         (
@@ -369,7 +444,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ac')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ac')
     @test.services('compute', 'network')
     def test_update_port_chain_add_flow_classifiers(self):
         (
@@ -433,7 +508,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ad')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ad')
     @test.services('compute', 'network')
     def test_update_port_chain_remove_flow_classifiers(self):
         (
@@ -497,7 +572,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ae')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ae')
     @test.services('compute', 'network')
     def test_update_port_chain_replace_flow_classifiers(self):
         (
@@ -561,7 +636,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21af')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21af')
     @test.services('compute', 'network')
     def test_update_port_chain_add_port_pair_groups(self):
         (
@@ -621,7 +696,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bf')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bf')
     @test.services('compute', 'network')
     def test_update_port_chain_remove_port_pair_groups(self):
         (
@@ -683,7 +758,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21be')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21be')
     @test.services('compute', 'network')
     def test_update_port_chain_replace_port_pair_groups(self):
         (
@@ -745,9 +820,9 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bc')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bc')
     @test.services('compute', 'network')
-    def test_update_port_chain_replace_port_pair_groups_flow_classifers(self):
+    def test_update_port_chain_replace_port_pair_groups_flow_classifiers(self):
         (
             server1_floating_ip, server1_port_id, server1_fixed_ip
         ) = self._setup_server(self.net1)
@@ -823,7 +898,7 @@ class TestSfc(base.SfcScenarioTest):
     def _wait_for_port_pair_group_status(self, port_pair_group, status):
         time.sleep(10)
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bb')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21bb')
     @test.services('compute', 'network')
     def test_update_port_pair_group_add_port_pairs(self):
         (
@@ -877,7 +952,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ba')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21ba')
     @test.services('compute', 'network')
     def test_update_port_pair_group_remove_port_pairs(self):
         (
@@ -928,7 +1003,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
     @test.services('compute', 'network')
     def test_update_port_pair_group_replace_port_pairs(self):
         (
@@ -979,7 +1054,7 @@ class TestSfc(base.SfcScenarioTest):
             username=self.ssh_user,
             private_key=self.keypair['private_key'])
 
-    @test.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
+    @decorators.idempotent_id('f970f6b3-6541-47ac-a9ea-f769be1e21b9')
     @test.services('compute', 'network')
     def test_multi_port_chains_update_port_pair_group_replace_port_pairs(
         self
@@ -1099,7 +1174,6 @@ class TestSfc(base.SfcScenarioTest):
         server = self.create_server(
             networks=[{'uuid': network['id']}],
             wait_until='ACTIVE',
-            wait_on_delete=False,
             **kwargs)
         waiters.wait_for_server_status(self.servers_client,
                                        server['id'], 'ACTIVE')
